/*
 * DEVICE LIBRARY TPIC6b595
 * Schieberegister - Softwareansteuerung - beliebiger Port
 * DKC2022
 */

//#include "config.h" //Allgemeine Projekt Config Header File
#include <avr/io.h>
#include "dev_tpic6b595.h"

//### --------- Low Level Functions -----------
static void dev_6b595_setbit(uint8_t bit)
{
    if (bit == 1)
        tpic6b595_PORT |= (1 << tpic6b595_PIN_SERIN);
    else
        tpic6b595_PORT &= ~(1 << tpic6b595_PIN_SERIN);
}

static void dev_6b595_clock(void)
{
    tpic6b595_PORT |= (1 << tpic6b595_PIN_CLOCK);
    asm("nop");
    tpic6b595_PORT &= ~(1 << tpic6b595_PIN_CLOCK);
}

//Gesetztes Byte ins Ausgaberegister laden
void dev_6b595_action(void)
{
    tpic6b595_PORT |= (1 << tpic6b595_PIN_RCK);
    asm("nop");
    tpic6b595_PORT &= ~(1 << tpic6b595_PIN_RCK);
}




//### --------- High Level Functions -----------

//Ein Byte im Schieberegister setzen
void dev_6b595_setbyte(uint8_t byte)
{
    uint8_t i
    for (i=0; i < 8; i++)
	{
		if( (byte << i) & 0x80 )
			dev_6b595_setbit(1);
		else
			dev_6b595_setbit(0);
		dev_6b595_clock();
	}
}

//Ein Byte setzen & ins Ausgabe register laden
void dev_6b595_setaload(uint8_t byte)
{
    dev_6b595_setbyte(byte);
    dev_6b595_action();
}

void dev_6b595_init(void)
{
    //IOs setzen
    tpic6b595_DDR |= (1 << tpic6b595_PIN_CLOCK) | (1 << pic6b595_PIN_RCK) | (1 << tpic6b595_PIN_SERIN);     //Ports auf Ausgang setzen
    tpic6b595_PORT &= ~((1 << tpic6b595_PIN_CLOCK) | (1 << pic6b595_PIN_RCK) | (1 << tpic6b595_PIN_SERIN)); //Pins auf Low

    //Alle Register auf definiert 0 setzen
    dev_6b595_setaload(0x00);
}

/* EXAMPLE in MAIN:
 * 
 * include "dev_tpic6b595.h"
 * 
 * main()
 * {
 *      dev_6b595_init();
 *      asm("nop");
 *      asm("nop");
 *      asm("nop");
 *      dev_6b595_setaload(0xAA);
 * 
 *      while(1);
 * }
 */
